<?php namespace Myriad\Squire;

class SquireImage extends SquireFile {

  /**
   * Sets the width of the image, in pixels.
   *
   * @param int $width the width in pixels
   *
   * @return $this
   */
  public function width($width) {
    return $this->param('w', $width);
  }

  /**
   * Sets the height of the image, in pixels.
   *
   * @param int $height the height in pixels
   *
   * @return $this
   */
  public function height($height) {
    return $this->param('h', $height);
  }

  /**
   * Sets how the image is fitted to its target dimensions.
   *
   * @param string $fit
   *
   * @return $this
   */
  public function fit($fit) {
    return $this->param('fit', $fit);
  }

  /**
   * Default. Resizes the image to fit within the width and height boundaries without cropping, distorting or altering
   * the aspect ratio.
   *
   * @return $this
   */
  public function fitContain() {
    return $this->fit('contain');
  }

  /**
   * Resizes the image to fit within the width and height boundaries without cropping, distorting or altering the
   * aspect ratio, and will also not increase the size of the image if it is smaller than the output size.
   *
   * @return $this
   */
  public function fitMax() {
    return $this->fit('max');
  }

  /**
   * Stretches the image to fit the constraining dimensions exactly. The resulting image will fill the dimensions, and
   * will not maintain the aspect ratio of the input image.
   *
   * @return $this
   */
  public function fitStretch() {
    return $this->fit('stretch');
  }

  /**
   * Resizes the image to fill the width and height boundaries and crops any excess image data. The resulting image
   * will match the width and height constraints without distorting the image.
   *
   * @return $this
   */
  public function fitCrop() {
    return $this->fit('crop');
  }

  /**
   * Sets where the image is cropped when the fit parameter is set to crop.
   *
   * @param $crop
   *
   * @return $this
   */
  public function crop($crop) {
    $this->fitCrop();
    return $this->param('crop', $crop);
  }

  /**
   * Crop the image from the top left.
   *
   * @return $this
   */
  public function cropTopLeft() {
    return $this->crop('top-left');
  }

  /**
   * Crop the image from the top.
   *
   * @return $this
   */
  public function cropTop() {
    return $this->crop('top');
  }

  /**
   * Crop the image from the top right.
   *
   * @return $this
   */
  public function cropTopRight() {
    return $this->crop('top-right');
  }

  /**
   * Crop the image from the left.
   *
   * @return $this
   */
  public function cropLeft() {
    return $this->crop('left');
  }

  /**
   * Crop the image from the center.
   *
   * @return $this
   */
  public function cropCenter() {
    return $this->crop('center');
  }

  /**
   * Crop the image from the right.
   *
   * @return $this
   */
  public function cropRight() {
    return $this->crop('right');
  }

  /**
   * Crop the image from the bottom left.
   *
   * @return $this
   */
  public function cropBottomLeft() {
    return $this->crop('bottom-left');
  }

  /**
   * Crop the image from the bottom.
   *
   * @return $this
   */
  public function cropBottom() {
    return $this->crop('bottom');
  }

  /**
   * Crop the image from the bottom right.
   *
   * @return $this
   */
  public function cropBottomRight() {
    return $this->crop('bottom-right');
  }

  /**
   * Crops the image to specific dimensions prior to any other resize operations.
   *
   * @param int $width
   * @param int $height
   * @param int $x
   * @param int $y
   *
   * @return $this
   */
  public function rect($width, $height, $x, $y) {
    return $this->param('rect', "$width,$height,$x,$y");
  }

  /**
   * Rotates the image.
   *
   * @param string $orientation
   *
   * @return $this
   */
  public function rotate($orientation) {
    return $this->param('or', $orientation);
  }

  /**
   * Automatically rotate the image based on Exif data.
   *
   * @return $this
   */
  public function rotateAuto() {
    return $this->rotate('auto');
  }

  /**
   * Rotate the image to 270 degrees.
   *
   * @return $this
   */
  public function rotateLeft() {
    return $this->rotate('270');
  }

  /**
   * Rotate the image to 90 degrees.
   *
   * @return $this
   */
  public function rotateRight() {
    return $this->rotate('90');
  }

  /**
   * Rotate the image 180 degrees.
   *
   * @return $this
   */
  public function rotateUpsideDown() {
    return $this->rotate('180');
  }

  /**
   * Adjusts the image brightness.
   *
   * @param int $brightness A value between -100 and +100, where 0 represents no change.
   *
   * @return $this
   */
  public function brightness($brightness) {
    return $this->param('bri', $brightness);
  }

  /**
   * Adjusts the image contrast.
   *
   * @param int $contrast A value between -100 and +100, where 0 represents no change.
   *
   * @return $this
   */
  public function contrast($contrast) {
    return $this->param('con', $contrast);
  }

  /**
   * Adjusts the image gamma.
   *
   * @param float $gamma a value between 0.1 and 9.99.
   *
   * @return $this
   */
  public function gamma($gamma) {
    return $this->param('gam', $gamma);
  }

  /**
   * Sharpen the image.
   *
   * @param int $sharpen a value between 0 and 100.
   *
   * @return $this
   */
  public function sharpen($sharpen) {
    return $this->param('sharp', $sharpen);
  }

  /**
   * Adds a blur effect to the image.
   *
   * @param int $blur a value between 0 and 100.
   *
   * @return $this
   */
  public function blur($blur) {
    return $this->param('blur', $blur);
  }

  /**
   * Applies a pixelation effect to the image.
   *
   * @param int $pixelate a value between 0 and 100.
   *
   * @return $this
   */
  public function pixel($pixelate) {
    return $this->param('pixel', $pixelate);
  }

  /**
   * Applies a filter effect to the image..
   *
   * @param string $filter grayscale or sepia.
   *
   * @return $this
   */
  public function filter($filter) {
    return $this->param('filt', $filter);
  }

  /**
   * Apply a grayscale filter.
   *
   * @return $this
   */
  public function grayscale() {
    return $this->filter('grayscale');
  }

  /**
   * Apply a sepia filter.
   *
   * @return $this
   */
  public function sepia() {
    return $this->filter('sepia');
  }

  /**
   * Defines the quality of the image. Note, this will enforce jpg output.
   *
   * @param int $quality a value between 0 and 100. Default: 90.
   *
   * @return $this
   */
  public function quality($quality) {
    $this->jpg();
    return $this->param('q', $quality);
  }

  /**
   * Encodes the image to a specific format.
   *
   * @param string $format jpg, png, or gif. Default: jpg.
   *
   * @return $this
   */
  public function format($format) {
    return $this->param('fm', $format);
  }

  /**
   * Formats the image as a jpg.
   *
   * @return $this
   */
  public function jpg() {
    return $this->format('jpg');
  }

  /**
   * Formats the image as a png.
   *
   * @return $this
   */
  public function png() {
    return $this->format('png');
  }

  /**
   * Formats the image as a gif.
   *
   * @return $this
   */
  public function gif() {
    return $this->format('gif');
  }

}